class InvisibleRoadsMacrosDiskError(Exception):
    pass


class PathValidationError(InvisibleRoadsMacrosDiskError):
    pass


class FileExtensionError(InvisibleRoadsMacrosDiskError):
    pass


class BadArchiveError(InvisibleRoadsMacrosDiskError):
    pass
